"use strict";
var express = require('express');

var app = express();

//make Sequelize a singleton
app.set('models', require('./components/models'));
var models = app.get('models');

var port = process.env.PORT || 3000;

var env         = process.env.NODE_ENV || "dev";
var config      = require("./components/config/server." + env + ".js");

//initialize logging
app.set('winston', require('winston'));
const winston = app.get('winston');
require('./components/config/logging')(winston);
var logger = winston.loggers.get('default');



app.listen(port, '0.0.0.0');
logger.info("Fresenius ASN API Server started on port " + port);

//initialize connections to the MQ framework
if (config.mqs && config.mqs === "rabbit") {
    var Rabbit = require("./components/helpers/RabbitHelper");
    app.set("mqs", new Rabbit(winston, config));
} else {
    var SquissHelper = require("./components/helpers/SqsHelper");
    app.set("mqs", new SquissHelper(winston, config));
}

//initialize express
require('./components/config/express')(app, config);

module.exports = app;
