USE [m3console]
GO

DROP TABLE [db_owner].[tbl_asn_address]
GO
DROP TABLE [db_owner].[tbl_asn]
GO

CREATE TABLE [db_owner].[tbl_asn] (
  [id] int IDENTITY(1,1) NOT NULL,
  [asn_id] nvarchar(255) NOT NULL,
  [asn_date] [datetime2](7) DEFAULT NULL,
  [clinic_id]	nvarchar(255)		,
  [expiration]	[DATETIME2](7)	DEFAULT NULL	,
  [invoice_number] nvarchar(255),
  [line_number]	nvarchar(255)		,
  [lot]	nvarchar(255)		,
  [material_description]	nvarchar(255)		,
  [ndc]	nvarchar(255)		,
  [po_number]	nvarchar(255)		,
  [po_date]	[DATETIME2](7)	DEFAULT NULL	,
  [sap_material_id]	nvarchar(255)		,
  [serialization_number]	nvarchar(255)		,
  [ship_from_id]	nvarchar(255)		,
  [ship_qty]	int		,
  [ship_via]	nvarchar(255)		,
  [tracking_id] nvarchar(255),
  [unit_price]	DECIMAL(9,2)		,
  [uom]	nvarchar(255)		,
  [vendor_id]	nvarchar(255)		,
  [vendor_material]	nvarchar(255)		,
  [vendor_name]	nvarchar(255)		,
  [received_qty] int DEFAULT -1,
  [lost_qty] int DEFAULT 0,
  PRIMARY KEY CLUSTERED
    (
      [id] ASC
    )WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO

CREATE NONCLUSTERED INDEX [idx_asn_clinic_nu] on [db_owner].[tbl_asn](clinic_id)
GO
CREATE NONCLUSTERED INDEX [idx_asn_date_nu] on [db_owner].[tbl_asn](asn_date)
GO

CREATE TABLE [db_owner].[tbl_asn_address] (
  [id] int IDENTITY(1,1) NOT NULL,
  [asn_id] INT NOT NULL,
  [ship_type] nvarchar(1) NOT NULL,
  [account_id] nvarchar(255),
  [name] nvarchar(255),
  [address] nvarchar(255),
  [city] nvarchar(255),
  [state] nvarchar(255),
  [zip] nvarchar(255) ,
PRIMARY KEY CLUSTERED
(
  [id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO

CREATE NONCLUSTERED INDEX [idx_fk_asn_address_asn_nu] on [db_owner].[tbl_asn_address](asn_id)
GO

ALTER TABLE [db_owner].[tbl_asn_address] ADD CONSTRAINT [FK_asn_address_asn] FOREIGN KEY ([asn_id]) REFERENCES [db_owner].[tbl_asn] ([id])
GO

DROP TABLE [db_owner].[tbl_asn_history]
GO
CREATE TABLE [db_owner].[tbl_asn_history] (
  [id] int IDENTITY(1,1) NOT NULL,
  [original_id] int NOT NULL,
  [asn_id] nvarchar(255) NOT NULL,
  [asn_date] [datetime2](7) DEFAULT NULL,
  [clinic_id]	nvarchar(255)		,
  [expiration]	[DATETIME2](7)	DEFAULT NULL	,
  [invoice_number] nvarchar(255),
  [line_number]	nvarchar(255)		,
  [lot]	nvarchar(255)		,
  [material_description]	nvarchar(255)		,
  [ndc]	nvarchar(255)		,
  [po_number]	nvarchar(255)		,
  [po_date]	[DATETIME2](7)	DEFAULT NULL	,
  [sap_material_id]	nvarchar(255)		,
  [serialization_number]	nvarchar(255)		,
  [ship_from_id]	nvarchar(255)		,
  [ship_qty]	int		,
  [ship_via]	nvarchar(255)		,
  [tracking_id] nvarchar(255),
  [unit_price]	DECIMAL(9,2)		,
  [uom]	nvarchar(255)		,
  [vendor_id]	nvarchar(255)		,
  [vendor_material]	nvarchar(255)		,
  [vendor_name]	nvarchar(255)		,
  [received_qty] int DEFAULT -1,
  [lost_qty] int DEFAULT 0,
  [date_archived]	[DATETIME2](7)	DEFAULT current_timestamp,
PRIMARY KEY CLUSTERED
(
  [id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX [idx_asn_hist_nu] on [db_owner].[tbl_asn_history](original_id)
GO

DROP TABLE [db_owner].[tbl_asn_address_history]
GO
CREATE TABLE [db_owner].[tbl_asn_address_history] (
  [id] int IDENTITY(1,1) NOT NULL,
  [original_id] int NOT NULL,
  [original_asn_id] INT NOT NULL,
  [ship_type] nvarchar(1) NOT NULL,
  [account_id] nvarchar(255),
  [name] nvarchar(255),
  [address] nvarchar(255),
  [city] nvarchar(255),
  [state] nvarchar(255),
  [zip] nvarchar(255) ,
  [date_archived]	[DATETIME2](7)	DEFAULT current_timestamp,
PRIMARY KEY CLUSTERED
(
  [id] ASC
)WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX [idx_asn_addr_hist_nu] on [db_owner].[tbl_asn_address_history](original_id)
GO


CREATE TABLE [db_owner].[tbl_inventory_asn] (
  [id] int IDENTITY(1,1) NOT NULL,
  [inventory_id] int NOT NULL,
  [asn_id] nvarchar(255) NOT NULL,
  [asn_date] [datetime2](7) DEFAULT NULL,
  [clinic_id]	nvarchar(255)		,
  [expiration]	[DATETIME2](7)	DEFAULT NULL	,
  [invoice_number] nvarchar(255),
  [line_number]	nvarchar(255)		,
  [lot]	nvarchar(255)		,
  [material_description]	nvarchar(255)		,
  [ndc]	nvarchar(255)		,
  [po_number]	nvarchar(255)		,
  [po_date]	[DATETIME2](7)	DEFAULT NULL	,
  [sap_material_id]	nvarchar(255)		,
  [serialization_number]	nvarchar(255)		,
  [ship_from_id]	nvarchar(255)		,
  [ship_qty]	int		,
  [ship_via]	nvarchar(255)		,
  [tracking_id] nvarchar(255),
  [unit_price]	DECIMAL(9,2)		,
  [uom]	nvarchar(255)		,
  [vendor_id]	nvarchar(255)		,
  [vendor_material]	nvarchar(255)		,
  [vendor_name]	nvarchar(255)		,
  [received_qty] int DEFAULT -1,
  [lost_qty] int DEFAULT 0,
  PRIMARY KEY CLUSTERED
    (
      [id] ASC
    )WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]

GO

CREATE UNIQUE INDEX [idx_fk_inventory_asn_uq] ON [db_owner].[tbl_inventory_asn] ([inventory_id])
GO
ALTER TABLE [db_owner].[tbl_inventory_asn] ADD CONSTRAINT [fk_inventory_asn_inventory] FOREIGN KEY ([inventory_id]) REFERENCES [db_owner].[tbl_inventory] ([id])
GO


CREATE TABLE [db_owner].[tbl_inventory_asn_history] (
  [id] int IDENTITY(1,1) NOT NULL,
  [original_id] int NOT NULL,
  [original_inventory_id] int NOT NULL,
  [asn_id] nvarchar(255) NOT NULL,
  [asn_date] [datetime2](7) DEFAULT NULL,
  [clinic_id]	nvarchar(255)		,
  [expiration]	[DATETIME2](7)	DEFAULT NULL	,
  [invoice_number] nvarchar(255),
  [line_number]	nvarchar(255)		,
  [lot]	nvarchar(255)		,
  [material_description]	nvarchar(255)		,
  [ndc]	nvarchar(255)		,
  [po_number]	nvarchar(255)		,
  [po_date]	[DATETIME2](7)	DEFAULT NULL	,
  [sap_material_id]	nvarchar(255)		,
  [serialization_number]	nvarchar(255)		,
  [ship_from_id]	nvarchar(255)		,
  [ship_qty]	int		,
  [ship_via]	nvarchar(255)		,
  [tracking_id] nvarchar(255),
  [unit_price]	DECIMAL(9,2)		,
  [uom]	nvarchar(255)		,
  [vendor_id]	nvarchar(255)		,
  [vendor_material]	nvarchar(255)		,
  [vendor_name]	nvarchar(255)		,
  [received_qty] int DEFAULT -1,
  [lost_qty] int DEFAULT 0,
  [date_archived]	[DATETIME2](7)	DEFAULT current_timestamp,
  PRIMARY KEY CLUSTERED
    (
      [id] ASC
    )WITH (PAD_INDEX = OFF, STATISTICS_NORECOMPUTE = OFF, IGNORE_DUP_KEY = OFF, ALLOW_ROW_LOCKS = ON, ALLOW_PAGE_LOCKS = ON) ON [PRIMARY]
) ON [PRIMARY]
GO

CREATE NONCLUSTERED INDEX [idx_inv_asn_hist_nu] on [db_owner].[tbl_inventory_asn_history](original_id)
GO


ALTER TABLE [db_owner].[tbl_inventory] ADD [asn_id] INT
GO
ALTER TABLE [db_owner].[tbl_inventory] ADD [modified_by] NVARCHAR(255) DEFAULT NULL
GO
ALTER TABLE [db_owner].[tbl_inventory] ADD [date_modified] [DATETIME2](7) DEFAULT NULL
GO
ALTER TABLE [db_owner].[tbl_inventory] ADD [sap_flag] BIT DEFAULT 0
GO
ALTER TABLE [db_owner].[tbl_inventory_history] ADD [asn_id] INT
GO
ALTER TABLE [db_owner].[tbl_inventory_history] ADD [modified_by] NVARCHAR(255) DEFAULT NULL
GO
ALTER TABLE [db_owner].[tbl_inventory_history] ADD [date_modified] [DATETIME2](7) DEFAULT NULL
GO
ALTER TABLE [db_owner].[tbl_inventory_history] ADD [sap_flag] BIT DEFAULT 0
GO
CREATE INDEX [idx_fk_inventory_asn_nu] ON [db_owner].[tbl_inventory] ([asn_id])
GO
ALTER TABLE [db_owner].[tbl_inventory] ADD CONSTRAINT [fk_inventory_asn] FOREIGN KEY ([asn_id]) REFERENCES [db_owner].[tbl_asn] ([id])
GO


DROP PROCEDURE [db_owner].[prc_insert_asn]
GO

CREATE PROCEDURE [db_owner].[prc_insert_asn] @inv_id INT, @asn_id INT, @recQty INT
AS
  BEGIN
    IF @asn_id IS NOT NULL
      BEGIN
        UPDATE [db_owner].[tbl_asn] SET [received_qty] = CASE WHEN [received_qty] < 0 THEN 0 ELSE [received_qty] END + @recQty WHERE [id] = @asn_id;
        UPDATE [db_owner].[tbl_asn] SET [lost_qty] = CASE WHEN ([ship_qty] - [received_qty]) < 0 THEN 0 ELSE ([ship_qty] - [received_qty]) END WHERE [id] = @asn_id;
        INSERT INTO [db_owner].[tbl_inventory_asn] ([inventory_id],[asn_id],[asn_date],[clinic_id],[expiration],[invoice_number],[line_number],[lot],[material_description],[ndc],[po_number],[po_date],[sap_material_id],[serialization_number],[ship_from_id],[ship_qty],[ship_via],[tracking_id],[unit_price],[uom],[vendor_id],[vendor_material],[vendor_name],[received_qty],[lost_qty])
          SELECT @inv_id,[asn_id],[asn_date],[clinic_id],[expiration],[invoice_number],[line_number],[lot],[material_description],[ndc],[po_number],[po_date],[sap_material_id],[serialization_number],[ship_from_id],[ship_qty],[ship_via],[tracking_id],[unit_price],[uom],[vendor_id],[vendor_material],[vendor_name],[received_qty],[lost_qty]
            FROM [db_owner].[tbl_asn] WHERE [db_owner].[tbl_asn].[id] = @asn_id;
      END
  END
GO

DROP PROCEDURE [db_owner].[prc_update_asn]
GO

CREATE PROCEDURE [db_owner].[prc_update_asn] @inv_id INT, @asn_id INT
AS
  BEGIN
    DECLARE @new_qty INT;
    DECLARE @new_lost INT;

    IF @asn_id IS NOT NULL
      BEGIN
        UPDATE [db_owner].[tbl_asn] SET [received_qty] = (SELECT SUM([count]) FROM [db_owner].[tbl_inventory] i WHERE i.asn_id is not null and i.asn_id = @asn_id) WHERE [id] = @asn_id;
        UPDATE [db_owner].[tbl_asn] SET [lost_qty] = CASE WHEN ([ship_qty] - [received_qty]) < 0 THEN 0 ELSE ([ship_qty] - [received_qty]) END WHERE [id] = @asn_id;
        SELECT @new_qty = received_qty, @new_lost = lost_qty FROM [db_owner].[tbl_asn] WHERE [db_owner].[tbl_asn].[id] = @asn_id;
        UPDATE [db_owner].[tbl_inventory_asn] SET received_qty = @new_qty, lost_qty = @new_lost WHERE [inventory_id] = @inv_id;
      END
  END
GO