DROP TABLE IF EXISTS `tbl_asn_address`;
DROP TABLE IF EXISTS `tbl_asn`;

CREATE TABLE IF NOT EXISTS `tbl_asn` (
	`id` int NOT NULL AUTO_INCREMENT,
	`asn_id` nvarchar(255) NOT NULL,
	`asn_date` TIMESTAMP(3) NULL DEFAULT NULL,
	`clinic_id`	nvarchar(255)		,
	`expiration`	DATETIME	DEFAULT NULL	,
	`invoice_number` nvarchar(255),
	`line_number`	nvarchar(255)		,
	`lot`	nvarchar(255)		,
	`material_description`	nvarchar(255)		,
	`ndc`	nvarchar(255)		,
	`po_number`	nvarchar(255)		,
	`po_date`	DATETIME	DEFAULT NULL	,
	`sap_material_id`	nvarchar(255)		,
	`serialization_number`	nvarchar(255)		,
	`ship_to_account`	nvarchar(255)		,
	`ship_to_name`	nvarchar(255)		,
	`ship_from_id`	nvarchar(255)		,
	`ship_qty`	int		,
	`ship_via`	nvarchar(255)		,
	`tracking_id` nvarchar(255),
	`unit_price`	DECIMAL(9,2)		,
	`uom`	nvarchar(255)		,
	`vendor_id`	nvarchar(255)		,
	`vendor_material`	nvarchar(255)		,
	`vendor_name`	nvarchar(255)		,
	`received_qty` int DEFAULT -1,
	`lost_qty` int DEFAULT 0,
PRIMARY KEY ( `id`)
) Engine InnoDB;

CREATE INDEX `idx_asn_clinic_nu` on `tbl_asn`(clinic_id);
CREATE INDEX `idx_asn_date_nu` on `tbl_asn`(asn_date);

CREATE TABLE IF NOT EXISTS `tbl_asn_address` (
	`id` int NOT NULL AUTO_INCREMENT,
	`asn_id` INT NOT NULL,
	`ship_type` nvarchar(1) NOT NULL,
	`account_id` nvarchar(255),
	`name` nvarchar(255),
	`address` nvarchar(255),
	`city` nvarchar(255),
	`state` nvarchar(255),
	`zip` nvarchar(255) ,
PRIMARY KEY ( `id`)
) Engine InnoDB;

CREATE INDEX `idx_fk_asn_address_asn_nu` on `tbl_asn_address`(asn_id);

ALTER TABLE `tbl_asn_address` ADD CONSTRAINT `FK_asn_address_asn` FOREIGN KEY (`asn_id`) REFERENCES `tbl_asn` (`id`);

DROP TABLE IF EXISTS `tbl_asn_history`;
CREATE TABLE IF NOT EXISTS `tbl_asn_history` (
	`id` int NOT NULL AUTO_INCREMENT,
	`original_id` int NOT NULL,
	`asn_id` nvarchar(255) NOT NULL,
	`asn_date` TIMESTAMP(3) NULL DEFAULT NULL,
	`clinic_id`	nvarchar(255)		,
	`expiration`	DATETIME	DEFAULT NULL	,
	`invoice_number` nvarchar(255),
	`line_number`	nvarchar(255)		,
	`lot`	nvarchar(255)		,
	`material_description`	nvarchar(255)		,
	`ndc`	nvarchar(255)		,
	`po_number`	nvarchar(255)		,
	`po_date`	DATETIME	DEFAULT NULL	,
	`sap_material_id`	nvarchar(255)		,
	`serialization_number`	nvarchar(255)		,
	`ship_to_account`	nvarchar(255)		,
	`ship_to_name`	nvarchar(255)		,
	`ship_from_id`	nvarchar(255)		,
	`ship_qty`	int		,
	`ship_via`	nvarchar(255)		,
	`tracking_id` nvarchar(255),
	`unit_price`	DECIMAL(9,2)		,
	`uom`	nvarchar(255)		,
	`vendor_id`	nvarchar(255)		,
	`vendor_material`	nvarchar(255)		,
	`vendor_name`	nvarchar(255)		,
	`received_qty` int DEFAULT -1,
	`lost_qty` int DEFAULT 0,
	PRIMARY KEY ( `id`)
) Engine InnoDB;

DROP TABLE IF EXISTS `tbl_asn_address_history`;
CREATE TABLE IF NOT EXISTS `tbl_asn_address_history` (
	`id` int NOT NULL AUTO_INCREMENT,
	`original_id` int NOT NULL,
	`original_asn_id` INT NOT NULL,
	`ship_type` nvarchar(1) NOT NULL,
	`account_id` nvarchar(255),
	`name` nvarchar(255),
	`address` nvarchar(255),
	`city` nvarchar(255),
	`state` nvarchar(255),
	`zip` nvarchar(255) ,
	PRIMARY KEY ( `id`)
) Engine InnoDB;

CREATE INDEX `idx_asn_addr_hist_nu` ON `tbl_asn_address_history`(`original_id`);

CREATE TABLE IF NOT EXISTS `tbl_inventory_asn` (
	`id` int NOT NULL AUTO_INCREMENT,
	`inventory_id` int NOT NULL,
	`asn_id` nvarchar(255) NOT NULL,
	`asn_date` TIMESTAMP(3) NULL DEFAULT NULL,
	`clinic_id`	nvarchar(255)		,
	`expiration`	DATETIME	DEFAULT NULL	,
	`invoice_number` nvarchar(255),
	`line_number`	nvarchar(255)		,
	`lot`	nvarchar(255)		,
	`material_description`	nvarchar(255)		,
	`ndc`	nvarchar(255)		,
	`po_number`	nvarchar(255)		,
	`po_date`	DATETIME	DEFAULT NULL	,
	`sap_material_id`	nvarchar(255)		,
	`serialization_number`	nvarchar(255)		,
	`ship_to_account`	nvarchar(255)		,
	`ship_to_name`	nvarchar(255)		,
	`ship_from_id`	nvarchar(255)		,
	`ship_qty`	int		,
	`ship_via`	nvarchar(255)		,
	`tracking_id` nvarchar(255),
	`unit_price`	DECIMAL(9,2)		,
	`uom`	nvarchar(255)		,
	`vendor_id`	nvarchar(255)		,
	`vendor_material`	nvarchar(255)		,
	`vendor_name`	nvarchar(255)		,
	`received_qty` int DEFAULT -1,
	`lost_qty` int DEFAULT 0,
	PRIMARY KEY ( `id`)
) Engine InnoDB;
CREATE UNIQUE INDEX `idx_fk_inventory_asn_uq` ON `tbl_inventory_asn` (`inventory_id`);
ALTER TABLE `tbl_inventory_asn` ADD CONSTRAINT `fk_inventory_asn_inventory` FOREIGN KEY (`inventory_id`) REFERENCES `tbl_inventory` (`id`);

CREATE TABLE IF NOT EXISTS `tbl_inventory_asn_history` (
	`id` int NOT NULL AUTO_INCREMENT,
	`original_id` int NOT NULL,
	`original_inventory_id` int NOT NULL,
	`asn_id` nvarchar(255) NOT NULL,
	`asn_date` TIMESTAMP(3) NULL DEFAULT NULL,
	`clinic_id`	nvarchar(255)		,
	`expiration`	DATETIME	DEFAULT NULL	,
	`invoice_number` nvarchar(255),
	`line_number`	nvarchar(255)		,
	`lot`	nvarchar(255)		,
	`material_description`	nvarchar(255)		,
	`ndc`	nvarchar(255)		,
	`po_number`	nvarchar(255)		,
	`po_date`	DATETIME	DEFAULT NULL	,
	`sap_material_id`	nvarchar(255)		,
	`serialization_number`	nvarchar(255)		,
	`ship_to_account`	nvarchar(255)		,
	`ship_to_name`	nvarchar(255)		,
	`ship_from_id`	nvarchar(255)		,
	`ship_qty`	int		,
	`ship_via`	nvarchar(255)		,
	`tracking_id` nvarchar(255),
	`unit_price`	DECIMAL(9,2)		,
	`uom`	nvarchar(255)		,
	`vendor_id`	nvarchar(255)		,
	`vendor_material`	nvarchar(255)		,
	`vendor_name`	nvarchar(255)		,
	`received_qty` int DEFAULT -1,
	`lost_qty` int DEFAULT 0,
	PRIMARY KEY ( `id`)
) Engine InnoDB;

CREATE INDEX `idx_inv_asn_hist_nu` ON `tbl_inventory_asn_history`(`original_id`);

ALTER TABLE `tbl_inventory` ADD (`asn_id` INT);
ALTER TABLE `tbl_inventory` ADD (`modified_by` NVARCHAR(255) DEFAULT NULL);
ALTER TABLE `tbl_inventory` ADD (`date_modified` DATETIME DEFAULT NULL);
ALTER TABLE `tbl_inventory` ADD (`sap_flag` TINYINT(1) DEFAULT 0);
ALTER TABLE `tbl_inventory_history` ADD (`asn_id` INT);
ALTER TABLE `tbl_inventory_history` ADD (`modified_by` NVARCHAR(255) DEFAULT NULL);
ALTER TABLE `tbl_inventory_history` ADD (`date_modified` DATETIME DEFAULT NULL);
ALTER TABLE `tbl_inventory_history` ADD (`sap_flag` TINYINT(1) DEFAULT 0);
CREATE INDEX `idx_fk_inventory_asn_nu` ON `tbl_inventory` (`asn_id`);
ALTER TABLE `tbl_inventory` ADD CONSTRAINT `fk_inventory_asn` FOREIGN KEY (`asn_id`) REFERENCES `tbl_asn` (`id`);

-- ALTER TABLE `tbl_clinic_profile` ADD (`status` NVARCHAR(15) DEFAULT 'open');
-- ALTER TABLE `tbl_clinic_profile` ADD (`discontinued` DATETIME DEFAULT NULL);

delimiter $$
-- DROP TRIGGER `trg_inventory_receipt_ins`
-- $$
CREATE TRIGGER `trg_inventory_receipt_ins` AFTER INSERT ON `tbl_inventory` FOR EACH ROW BEGIN
	IF NEW.asn_id IS NOT NULL THEN
		UPDATE tbl_asn SET received_qty = CASE WHEN received_qty < 0 THEN 0 ELSE received_qty END + NEW.`count` WHERE tbl_asn.ID = NEW.asn_id;
		UPDATE tbl_asn SET lost_qty = CASE WHEN ship_qty - received_qty < 0 THEN 0 ELSE ship_qty - received_qty END WHERE tbl_asn.ID = NEW.asn_id;
		INSERT INTO tbl_inventory_asn (`inventory_id`,`asn_id`,`asn_date`,`clinic_id`,`expiration`,`invoice_number`,`line_number`,`lot`,`material_description`,`ndc`,`po_number`,`po_date`,`sap_material_id`,`serialization_number`,`ship_to_account`,
																	 `ship_to_name`,
																	 `ship_from_id`,
																	 `ship_qty`,
																	 `ship_via`,
																	 `tracking_id`,
																	 `unit_price`,
																	 `uom`,
																	 `vendor_id`,
																	 `vendor_material`,
																	 `vendor_name`,
																	 `received_qty`,
																	 `lost_qty`)
			SELECT NEW.id,`asn_id`,`asn_date`,`clinic_id`,`expiration`,`invoice_number`,`line_number`,`lot`,`material_description`,`ndc`,`po_number`,`po_date`,NEW.`sap_material`,`serialization_number`,`ship_to_account`,
				`ship_to_name`,
				`ship_from_id`,
				`ship_qty`,
				`ship_via`,
				`tracking_id`,
				`unit_price`,
				`uom`,
				`vendor_id`,
				`vendor_material`,
				`vendor_name`,
				`received_qty`,
				`lost_qty`
			FROM tbl_asn WHERE tbl_asn.ID = NEW.asn_id;
	END IF;
END
$$

-- DROP TRIGGER `trg_inventory_receipt_upd`
-- $$
CREATE TRIGGER trg_inventory_receipt_upd AFTER UPDATE ON tbl_inventory FOR EACH ROW BEGIN
	DECLARE new_qty INT;
	DECLARE new_lost INT;

	IF NEW.asn_id IS NOT NULL THEN
			UPDATE tbl_asn SET received_qty = received_qty + (NEW.`count` - OLD.`count`)
			WHERE ID = NEW.asn_id;
			UPDATE tbl_asn SET lost_qty = CASE WHEN ship_qty - received_qty < 0 THEN 0 ELSE ship_qty - received_qty END
			WHERE ID = NEW.asn_id;
			SELECT received_qty, lost_qty
				INTO @new_qty, @new_lost
		    FROM tbl_asn
			 WHERE ID = NEW.asn_id
		   LIMIT 1;
		  UPDATE tbl_inventory_asn SET received_qty = @new_qty, lost_qty = @new_lost WHERE inventory_id = NEW.id;
		END IF;
END;
$$